
CREATE OR REPLACE FUNCTION spJnl_Edit (
	_JnlNo		VARCHAR(15),
	_AccOld		VARCHAR(10),
	_AccNew		VARCHAR(10)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

BEGIN

	SELECT INTO _spret * FROM spJnl_LgrVoid(_JnlNo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	UPDATE jnld SET accno=_AccNew WHERE jnlno=_JnlNo AND accno=_AccOld;

	SELECT INTO _spret * FROM spJnl_New(_JnlNo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION spJnl_EditByRef (
	_RefType	VARCHAR(5),
	_RefNo		VARCHAR(15),
	_AccOld		VARCHAR(10),
	_AccNew		VARCHAR(10)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_JnlNo		VARCHAR;
	cJnl		REFCURSOR;
BEGIN
	OPEN cJnl FOR
		SELECT JnlNo FROM Jnl WHERE RefType=_RefType AND RefNo=_RefNo;
	LOOP
		FETCH cJnl INTO _JnlNo;
		EXIT WHEN NOT FOUND;

		SELECT INTO _spret * FROM spJnl_LgrVoid(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		UPDATE jnld SET accno=_AccNew WHERE jnlno=_JnlNo AND accno=_AccOld;

		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

	END LOOP;
	CLOSE cJnl;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION spJnl_LgrVoid (
	_JnlNo		VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	cAcc		REFCURSOR;
	_AccNo		VARCHAR(10);
	_tgl		DATE;
	_amount		NUMERIC(19,4);
	_DK		VARCHAR(1);
	_DefDK		VARCHAR(1);
	_isAutoGen	BOOLEAN;
	_LockDate	DATE;
	_BranchID	VARCHAR(5);
	_DeptID		VARCHAR(5);
	_PrjID		VARCHAR(5);
	_PerStart	DATE;
BEGIN
	_retval.ErrNo:=50001;
	_retval.ErrMsg:='General Error (spJnl_Void)';

	SELECT INTO 
		_tgl, _isAutoGen
		jnldate, isAutoGen
	FROM jnl WHERE JnlNo=_JnlNo;

	SELECT INTO _PerStart perstart from per where active=true;
	IF(_PerStart>_tgl) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Transaksi Melebihi Batas Periode Akuntansi ! (spJnl_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='JNL';
	IF(_LockDate is not null AND _LockDate>_tgl) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Jurnal Melebihi Batas Tanggal Kunci Transaksi ! (spJnl_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	OPEN cAcc FOR SELECT d.accno,d.dk,d.jnldamt,j.branchid,d.deptid,d.prjid FROM jnl j, jnld d WHERE j.jnlno=d.jnlno
	AND d.JnlNo=_JnlNo;
	LOOP
		FETCH cAcc INTO _AccNo,_DK,_amount,_BranchID,_DeptID,_PrjID;
		EXIT WHEN NOT FOUND;
		SELECT INTO _DefDK accdk FROM acc WHERE AccNo=_AccNo;
		IF _DefDK=_DK THEN	
			_amount:=_amount*-1;
		END IF;			
		
		SELECT INTO _spret * FROM spAccBal_Update(_AccNo,_tgl,_amount,_BranchID,_DeptID,_PrjID);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
	END LOOP;

	/** Void nested trans: BPA, CashA */

	IF (_isAutoGen=false) THEN
		SELECT INTO _spret * FROM spBPA_Void_ByRef('JNL', _JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _spret * FROM spCashA_Void_ByRef('JNL', _JnlNo, NULL, NULL);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;		
	END IF;
	/***/

	_retval:=(50010, 'Error Delete Lgr (spJnl_Void)');
	DELETE FROM Lgr WHERE jnlno=_JnlNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION spJnl_LgrVoidByRef (
	_RefType	VARCHAR(5),
	_RefNo		VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_JnlNo		VARCHAR;
	cJnl		REFCURSOR;
BEGIN
	OPEN cJnl FOR
		SELECT JnlNo FROM Jnl WHERE RefType=_RefType AND RefNo=_RefNo;
	LOOP
		FETCH cJnl INTO _JnlNo;
		EXIT WHEN NOT FOUND;

		SELECT INTO _spret * FROM spJnl_LgrVoid(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cJnl;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spFix_BPAccJnl () RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_bpID		VARCHAR;
	_bpAccCount	INT;
	_bpAccAP	VARCHAR;
	_bpAccAR	VARCHAR;

	_saleNo		VARCHAR;
	_purcNo		VARCHAR;
	_rcvNo		VARCHAR;
	_payNo		VARCHAR;

	cBPAccAP	REFCURSOR;
	cBPAccAR	REFCURSOR;

	cSale		REFCURSOR;
	_saleAccAR	VARCHAR;
	_saleCrcID	VARCHAR;

	cRcv		REFCURSOR;
	_rcvJnlNo	VARCHAR;
	_rcvJnlAccAR	VARCHAR;

	cPurc		REFCURSOR;
	_purcAccAP	VARCHAR;
	_purcCrcID	VARCHAR;

	cPay		REFCURSOR;
	_payJnlNo	VARCHAR;
	_payJnlAccAP	VARCHAR;
	
BEGIN
/* Fixing Sale & Rcv */
	OPEN cBPAccAR FOR
		SELECT bpid FROM bpacc GROUP BY bpid HAVING count(DISTINCT accar) > 1 ORDER BY bpid;
	LOOP
		FETCH cBPAccAR INTO _bpID;
		EXIT WHEN NOT FOUND;

		OPEN cSale FOR
			SELECT saleno,crcid,accar FROM sale WHERE custid=_bpID;
		LOOP
			FETCH cSale INTO _saleNo, _saleCrcID, _saleAccAR;
			EXIT WHEN NOT FOUND;

			SELECT INTO _bpAccAR accar FROM bpacc WHERE bpid=_bpID AND crcid=_saleCrcID;

			IF(_bpAccAR<>_saleAccAR) THEN
				UPDATE sale SET accar=_bpAccAr WHERE saleno=_saleNo;

				SELECT INTO _spret * FROM spJnl_EditByRef('SALE', _saleNo, _saleAccAR, _bpAccAR);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;

			

			OPEN cRcv FOR
				SELECT rcvno FROM spaid WHERE refno=_saleNo and reftype='SALE';
			LOOP
				FETCH cRcv INTO _rcvNo;
				EXIT WHEN NOT FOUND;

				SELECT INTO _rcvJnlNo JnlNo FROM Jnl WHERE RefType='RCV' AND RefNo=_rcvNo 
				AND UPPER(jnlnote) LIKE UPPER('%Penerimaan%');

				SELECT INTO _rcvJnlAccAR accno FROM JnlD WHERE JnlNo=_rcvJnlNo AND AccNo=_saleAccAR;
				
				IF(_bpAccAR<>_rcvJnlAccAR) THEN
					SELECT INTO _spret * FROM spJnl_EditByRef('RCV', _rcvNo, _rcvJnlAccAR, _bpAccAR);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;
				END IF;

			END LOOP;
			CLOSE cRcv;

		END LOOP;
		CLOSE cSale;

	END LOOP;
	CLOSE cBPAccAR;

/* Fixing Purc & Pay */
	OPEN cBPAccAP FOR
		SELECT bpid FROM bpacc GROUP BY bpid HAVING count(DISTINCT accap) > 1 ORDER BY bpid;
	LOOP
		FETCH cBPAccAP INTO _bpID;
		EXIT WHEN NOT FOUND;

		OPEN cPurc FOR
			SELECT purcno,crcid,accap FROM purc WHERE vendorid=_bpID;
		LOOP
			FETCH cPurc INTO _purcNo, _purcCrcID, _purcAccAP;
			EXIT WHEN NOT FOUND;

			SELECT INTO _bpAccAP accap FROM bpacc WHERE bpid=_bpID AND crcid=_purcCrcID;

			IF(_bpAccAP<>_purcAccAP) THEN
				UPDATE purc SET accap=_bpAccAP WHERE purcno=_purcNo;
			
				SELECT INTO _spret * FROM spJnl_EditByRef('PURC', _purcNo, _purcAccAP, _bpAccAP);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;	
			END IF;

			OPEN cPay FOR
				SELECT payno FROM ppaid WHERE refno=_purcNo and reftype='PURC';
			LOOP
				FETCH cPay INTO _payNo;
				EXIT WHEN NOT FOUND;

				SELECT INTO _payJnlNo JnlNo FROM Jnl WHERE RefType='PAY' AND RefNo=_payNo 
				AND UPPER(jnlnote) LIKE UPPER('%Pembayaran%');

				SELECT INTO _payJnlAccAP accno FROM JnlD WHERE JnlNo=_payJnlNo AND AccNo=_purcAccAP;
				
				IF(_bpAccAP<>_payJnlAccAP) THEN
					SELECT INTO _spret * FROM spJnl_EditByRef('PAY', _payNo, _payJnlAccAP, _bpAccAP);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;
				END IF;

			END LOOP;
			CLOSE cPay;

		END LOOP;
		CLOSE cPurc;

	END LOOP;
	CLOSE cBPAccAP;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;

SELECT * FROM spFix_BPAccJnl();
